/**
    @file ft51_i2c_master.c

    @brief I2C Master Library API functions.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include "ft51.h"
#include "registers/ft51_registers.h"

// I2C #defines
#define I2C_STATUS_IDLE         0x20
#define I2C_STATUS_ADDRACK      0x04
#define I2C_STATUS_ERROR        0x02
#define I2C_STATUS_BUSY         0x01
#define I2C_FLAGS_RUN           0x01
#define I2C_FLAGS_START         0x02
#define I2C_FLAGS_STOP          0x04
#define I2C_FLAGS_ACK           0x08
#define I2C_FLAGS_HS            0x10
#define I2C_FLAGS_ADDR          0x20
#define I2C_FLAGS_SLRST         0x40
#define I2C_FLAGS_RSTB          0x80
#define I2C_READ_NOT_WRITE      0x01
#define I2C_TIMER_HS_SELECT     0x80
#define I2C_TIMER_PERIOD_MASK   0x7F

static uint8_t i2c_wait_for()
{
    uint8_t status;
    __asm__ ("NOP");
    __asm__ ("NOP");
    __asm__ ("NOP");
    __asm__ ("NOP");
    __asm__ ("NOP");
    __asm__ ("NOP");

    do
    { // loop while busy
        status = I2CMSR;
        if (!(status & I2C_STATUS_BUSY))
        {
            break;
        }
    } while(1);

    if (status & I2C_STATUS_ERROR)
        return 0;
    else
        return 1;
}

// I2C Macros
#define I2C_START(address, data)                                 \
    I2CMSA = address;                                            \
    I2CMBUF = data;                                              \
    I2CMCR =  I2C_FLAGS_START | I2C_FLAGS_RUN;                   \
    i2c_wait_for();

#define I2C_START_END(address, data)                             \
    I2CMSA = address;                                            \
    I2CMBUF = data;                                              \
    I2CMCR =  I2C_FLAGS_START | I2C_FLAGS_RUN | I2C_FLAGS_STOP;  \
    i2c_wait_for();

#define I2C_OUT(data)                                            \
    I2CMBUF = data;                                              \
    I2CMCR = I2C_FLAGS_RUN;                                      \
    i2c_wait_for();

#define I2C_IN(data)                                             \
    I2CMCR = I2C_FLAGS_RUN;                                      \
    i2c_wait_for();                                              \
    data = I2CMBUF;

#define I2C_END(data)                                            \
    I2CMBUF = data;                                              \
    I2CMCR = I2C_FLAGS_RUN | I2C_FLAGS_STOP;                     \
    i2c_wait_for();

void I2C_master_initialise(void)
{
    // Reset I2C Master block
    I2CMCR |= I2C_FLAGS_RSTB;
    __asm__ ("NOP");

    // Freq
    I2CMTP = 0x40;
    __asm__ ("NOP");

    // Set dummy I2C Slave Address
    // Bit[0] = R/!S
    I2CMSA = 0x22<<1;
}

void I2C_master_write(const uint8_t addr, const uint8_t cmd,
        const uint8_t *data, uint16_t number_to_write)
{
    I2C_START(addr, cmd);
    if (number_to_write--)
    {
        while (number_to_write--)
        {
            I2C_OUT(*data++);
        };
    }
    I2C_END(*data);
}

void I2C_master_read(const uint8_t addr, const uint8_t cmd,
        uint8_t *data, uint16_t number_to_read)
{
    I2C_START(addr, cmd);
    I2C_START((addr | 0x01), 0x00);
    if (number_to_read--)
    {
        while (number_to_read--)
        {
            I2C_IN(*data++);
        };
    }
    I2C_END(*data);
}

uint8_t I2C_master_get_status(void)
{
    return I2CMSR;
}

void I2C_master_set_timer(uint8_t hs, uint8_t timer)
{
    I2CMTP = (hs!=0)?I2C_TIMER_HS_SELECT:0 | timer;
}