/**
    @file ft51_usb_isr.c

    @brief USB Library API interrupt handler function.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include <stdlib.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

static void USB_ISR_helper(uint8_t ep_index)
{
    USB_endpoint *ep = &USB_ep[ep_index];

    FT122_CMD = READ_ENDPOINT_STATUS + ep_index;
    ep->status = FT122_DATA; // Read EP status.
    FT122_CMD = READ_LAST_TRANSACTION_STATUS + ep_index;
    ep->last_status = FT122_DATA; // Read Last transaction status.
}

/**
 @brief   \par USB Interrupt Service Routine
 @details Called when an interrupt is received from USB.  This function
 determines the source of the interrupt and sets bits within the
 data structures accordingly.  For reset and suspend it will call
 the appropriate callback function if specified.
 @note    Please refer to section 6.3.1 of the FT122 Data Sheet for more
 information.
 **/
void USB_ISR(const uint8_t flags)
{
    uint8_t int_reg[4];

    (void) flags; // Suppress warning about unused parameter.

    FT122_CMD = RD_INTERRUPT_REG;
    int_reg[0] = FT122_DATA;
    int_reg[1] = FT122_DATA;
#if USB_MAX_ENDPOINT_COUNT > USB_MAX_ENDPOINT_COUNT_0_TO_2
    int_reg[2] = FT122_DATA;
#endif // USB_MAX_ENDPOINT_COUNT
#if USB_MAX_ENDPOINT_COUNT > USB_MAX_ENDPOINT_COUNT_0_TO_6
    int_reg[3] = FT122_DATA;
#endif // USB_MAX_ENDPOINT_COUNT
    if (int_reg[0])
    {
        if (int_reg[0] & CTRL_OUT_INT)
        {
            USB_ISR_helper(CTRL_OUT);
        }

        if (int_reg[0] & CTRL_IN_INT)
        {
            USB_ISR_helper(CTRL_IN);
        }

#if USB_MAX_ENDPOINT_COUNT > USB_MAX_ENDPOINT_COUNT_CONTROL
        if (int_reg[0] & EP_1_OUT_INT)
        {
            USB_ISR_helper(EP_1_OUT);
        }

        if (int_reg[0] & EP_1_IN_INT)
        {
            USB_ISR_helper(EP_1_IN);
        }

        if (int_reg[0] & EP_2_OUT_INT)
        {
            USB_ISR_helper(EP_2_OUT);
        }

        if (int_reg[0] & EP_2_IN_INT)
        {
            USB_ISR_helper(EP_2_IN);
        }
#endif // USB_MAX_ENDPOINT_COUNT
        if (int_reg[0] & SUSPEND_CHANGE_INT)
        {
            USB_suspend_changed++;
        }
        if (int_reg[0] & BUS_RESET_INT)
        {
            USB_reset_signal = 1;
        }
    }
    if (int_reg[1] & LPM_CHANGE_INT)
    {
        USB_LPM_changed++; // Set this bit to indicate LPM-triggered transition from L0 (Active) to L1 (Sleep).
    }

#if USB_MAX_ENDPOINT_COUNT > USB_MAX_ENDPOINT_COUNT_0_TO_2
    if (int_reg[2])
    {
        if (int_reg[2] & EP_3_OUT_INT)
        {
            USB_ISR_helper(EP_3_OUT);
        }
        if (int_reg[2] & EP_3_IN_INT)
        {
            USB_ISR_helper(EP_3_IN);
        }
        if (int_reg[2] & EP_4_OUT_INT)
        {
            USB_ISR_helper(EP_4_OUT);
        }
        if (int_reg[2] & EP_4_IN_INT)
        {
            USB_ISR_helper(EP_4_IN);
        }
        if (int_reg[2] & EP_5_OUT_INT)
        {
            USB_ISR_helper(EP_5_OUT);
        }
        if (int_reg[2] & EP_5_IN_INT)
        {
            USB_ISR_helper(EP_5_IN);
        }
        if (int_reg[2] & EP_6_OUT_INT)
        {
            USB_ISR_helper(EP_6_OUT);
        }
        if (int_reg[2] & EP_6_IN_INT)
        {
            USB_ISR_helper(EP_6_IN);
        }
    }
#endif // USB_MAX_ENDPOINT_COUNT
#if USB_MAX_ENDPOINT_COUNT > USB_MAX_ENDPOINT_COUNT_0_TO_6
    if (int_reg[3])
    {
        if (int_reg[3] & EP_7_OUT_INT)
        {
            if (USB_ep[EP_7_OUT].max_packet_size)
            {
                USB_ISR_helper(EP_7_OUT);
            }
        }
        if (int_reg[3] & EP_7_IN_INT)
        {
            if (USB_ep[EP_7_IN].max_packet_size)
            {
                USB_ISR_helper(EP_7_IN);
            }
        }
    }
#endif // USB_MAX_ENDPOINT_COUNT
}
