/**
    @file ft51_usb_process.c

    @brief USB Library API task processing function.

    @version 1.0.0

**/
/*
 * ============================================================================
 * History
 * =======
 *
 * (C) Copyright 2014, Future Technology Devices International Ltd.
 * ============================================================================
 *
 * This source code ("the Software") is provided by Future Technology Devices
 * International Limited ("FTDI") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of FTDI liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on FTDI's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, FTDI
 * has no liability in relation to those amendments.
 * ============================================================================
 */
#include <stdint.h>
#include <stdlib.h>
#include "ft51.h"
#include "ft51_sfr.h"
#include "ft51_general_config.h"
#include "registers/ft51_registers.h"
#include "ft51_usb.h"
#include "ft51_usb_internal.h"

/**
 * @brief	Enable USB compliance when hub enabled
 * @details Will turn on feature to turn of clocks when USB hub is
 * 			suspended.
 **/
#ifndef USB_COMPLIANCE
#define USB_COMPLIANCE 0
#endif

/**
 @brief   Process USB SETUP packets
 @details Read in a SETUP request token from the USB device and enact any
 callbacks for the standard, class and vendor request types.
 **/
static void USB_process_setup_packet(void)
{
    USB_request_callback cb = NULL;
    uint8_t data;
    uint8_t i;
    FT51_STATUS status = FT51_FAILED;

    DISABLE_INTERRUPTS;

    FT122_CMD = SELECT_ENDPOINT + CTRL_OUT;
    FT122_CMD = ACK_SETUP;
    FT122_CMD = RD_BUFFER;

    data = FT122_DATA;  // First byte returned is reserved
    data = FT122_DATA;  // Second byte is length of data buffer

    if(data != 8)
	{
    	// Request is unrecognised, or handler failed.
        FT122_CMD = CLR_BUFFER;
		USB_stall_endpoint(USB_EP_0, USB_DIR_OUT);
		USB_stall_endpoint(USB_EP_0, USB_DIR_IN);
		return;
	}

    for (i = 0; i < data; i++)
    {
        USB_received_setup_packet[i] = FT122_DATA;
    }

    FT122_CMD = CLR_BUFFER;

    // FT122 data sheet, 6.3.10: Must send ACK to Ctrl In too.
    FT122_CMD = SELECT_ENDPOINT + CTRL_IN;
    FT122_CMD = ACK_SETUP;
    FT122_CMD = CLR_BUFFER;

    ENABLE_INTERRUPTS;

    // Invoke request-specific handler, if any; otherwise, stall.
    if (USB_standard_req_cb != NULL
            && (USB_received_setup_packet[0] & USB_BMREQUESTTYPE_TYPE_MASK)
                    == USB_BMREQUESTTYPE_STANDARD)
    {
        cb = USB_standard_req_cb;
    }
    else if (USB_class_req_cb != NULL
            && (USB_received_setup_packet[0] & USB_BMREQUESTTYPE_TYPE_MASK)
                    == USB_BMREQUESTTYPE_CLASS)
    {
        cb = USB_class_req_cb;
    }
    else if (USB_vendor_req_cb != NULL
            && (USB_received_setup_packet[0] & USB_BMREQUESTTYPE_TYPE_MASK)
                    == USB_BMREQUESTTYPE_VENDOR)
    {
        cb = USB_vendor_req_cb;
    }

    if (cb != NULL)
    {
        status = cb((USB_device_request *) USB_received_setup_packet);
    }

    if (status != FT51_OK)
    {
        // Request is unrecognised, or handler failed.
        USB_stall_endpoint(USB_EP_0, USB_DIR_OUT);
        USB_stall_endpoint(USB_EP_0, USB_DIR_IN);
    }
}

/**
 @brief   \par Process USB packets
 @details Check for received SETUP requests from the USB device and
 manage the control endpoint modes.
 **/
uint8_t USB_process(void)
{
    uint8_t process = 0;
    uint8_t status = 0;

    if (USB_reset_signal)
    {
        DISABLE_INTERRUPTS;
        // Clear signal from ISR
        USB_reset_signal = 0;
        USB_suspend_changed = 0;
        ENABLE_INTERRUPTS;
        // Revert remote wakeup enable to default value of off
        USB_clear_remote_wakeup();

        USB_current_state = DEFAULT;
        if (USB_reset_cb)
            USB_reset_cb(0);

        USB_core_suspend_request = 0;
    }

    if (USB_suspend_changed)
    {
        DISABLE_INTERRUPTS;
        // Remove signal from ISR
        USB_suspend_changed--;
        ENABLE_INTERRUPTS;

        if (USB_current_state < SUSPENDED)
        {
            // USB 2.0 section 7.1.7.6 says any Powered state 
            // may transition to Suspended.
            if (USB_current_state >= POWERED)
            {
                // Push current state to top nibble of variable
                USB_current_state <<= 4;
                // Mark low nibble as suspended therefore
                USB_current_state |= SUSPENDED;

                if (USB_suspend_cb)
                {
                    USB_suspend_cb(0);
                }

                if ((USB_hub_config & MASK_HUB_ENABLE) == 0)
                {
                    // Hub is not enabled, we can move to low-power if required.
                    USB_core_suspend_request = 1;
                }
                else
                {
                    // Hub is enabled, we need to wait until it is suspended
                    // before moving to low-power.
                    USB_core_suspend_request = -1;                  
                }
            }
        }
        else
        {
            USB_core_suspend_request = 0;

            USB_wakeup();

            USB_current_state >>= 4;
            if (USB_resume_cb)
            {
                USB_resume_cb(0);
            }
        }
    }

    DISABLE_INTERRUPTS;
    if (USB_ep[CTRL_OUT].last_status)
    {
        process = 1;
        USB_ep[CTRL_OUT].last_status = 0;
    }
    ENABLE_INTERRUPTS;

    if (process)
    {
        // Host has sent OUT a control packet.
        if ((USB_ep[CTRL_OUT].status) & SETUP_PACKET)
        {
            USB_process_setup_packet();
        }
        else
        {
            // A data OUT packet terminates any ongoing IN transfers so
            // clear the flag in case a WR_BUFFER is taking place
            DISABLE_INTERRUPTS;
            FT122_CMD = SELECT_ENDPOINT + CTRL_OUT;
            FT122_CMD = CLR_BUFFER;
            ENABLE_INTERRUPTS;
        }
    }

    DISABLE_INTERRUPTS;
    if (USB_ep[CTRL_IN].last_status)
    {
        // Host has read IN contents of our control IN buffer,
        // so clear it now.
        FT122_CMD = SELECT_ENDPOINT + CTRL_IN;
        FT122_CMD = CLR_BUFFER;
        USB_ep[CTRL_IN].last_status = 0;
    }
    ENABLE_INTERRUPTS;

    if (USB_LPM_changed)
    {
        // FIXME implement callback
        DISABLE_INTERRUPTS;
        // Remove signal from ISR
        USB_LPM_changed--;
        ENABLE_INTERRUPTS;
    }

    // Suspend the device if requested to by USB host
    if (USB_core_suspend_request == 1)
    {
        USB_core_suspend_request = 0;

        if ((USB_hub_config & MASK_HUB_EXT_LOCALPWRSRC) == 0)
        {
            // Bus powered... stop clocks and suspend whole
            // device to save power.
            device_stop_request();

            USB_wakeup();
        }
    }
    else if (USB_core_suspend_request == -1)
    {
#if (USB_COMPLIANCE)
        USB_core_suspend_request = 1;
#else
        // Poll hub status
        USB_GET_HUB_ENABLE(status);
        if (status == 0)
        {
            // If HUB_ENABLE is zero, the hub is suspended and we can
            // optionally stop the device clocks using STOP_REQUEST.
            USB_core_suspend_request = 1;
        }
#endif
    }

    return process;
}
